from Crypto.Cipher import AES
from Crypto.Random import get_random_bytes
import hashlib


def encrypt_text(text: str, secret_key: str):
    secret_key = secret_key.zfill(16)
    nonce = get_random_bytes(8)
    secret_key_to_bytes = secret_key.encode('utf-8')
    cipher = AES.new(secret_key_to_bytes, AES.MODE_CTR, nonce=nonce)
    encrypted_text = cipher.encrypt(text.encode('utf-8'))
    return nonce.hex() + encrypted_text.hex()


def decrypt_text(encrypted_text, secret_key):
    nonce = bytes.fromhex(encrypted_text[:16])
    secret_key = secret_key.zfill(16)
    encrypted_bytes = bytes.fromhex(encrypted_text[16:])
    secret_key_to_bytes = secret_key.encode('utf-8')
    cipher = AES.new(secret_key_to_bytes, AES.MODE_CTR, nonce=nonce)
    decrypted_text = cipher.decrypt(encrypted_bytes)
    return decrypted_text.decode('utf-8')


def hash_password(password):
    password_bytes = password.encode('utf-8')
    sha256_hash = hashlib.sha256()
    sha256_hash.update(password_bytes)
    hashed_password = sha256_hash.hexdigest()
    return hashed_password


def validate_password(password, hash):
    return hash_password(password) == hash



import base64
import hashlib
import hmac
import time

class TOTPGenerator:
    def __init__(self, seed: str, algorithm: str = "sha1", period: int = 30, digits: int = 6, **kwargs):
        try:
            if algorithm.lower() not in ["sha1", "sha256", "sha512"]:
                raise ValueError("Invalid algorithm. Must be one of 'sha1', 'sha256', or 'sha512'")
            if digits not in [6, 8]:
                raise ValueError("Invalid digits. Must be either 6 or 8")
            if period <= 0:
                raise ValueError("Invalid period. Must be greater than 0")
            if len(seed) % 8 != 0:
                seed = seed + "=" * (8 - len(seed) % 8)

            self.seed = seed
            self.algorithm = algorithm.lower()
            self.period = period
            self.digits = digits

        except ValueError as error:
            print(error)
    
    def generateTOTP(self):
        try:                
            # Decode the seed from base32 to obtain the key
            key = base64.b32decode(self.seed.upper())

            # Get the current time in intervals of given period
            now = int(time.time() // self.period)

            # Convert the current time to a byte string of length 8
            time_in_bytes = now.to_bytes(8, "big")

            # Create an HMAC object using the key and time byte string, with given algorithm
            if self.algorithm == "sha1":
                hashing = hmac.new(key, time_in_bytes, hashlib.sha1)
            elif self.algorithm == "sha256":
                hashing = hmac.new(key, time_in_bytes, hashlib.sha256)
            elif self.algorithm == "sha512":
                hashing = hmac.new(key, time_in_bytes, hashlib.sha512)

            # Calculate the digest of the HMAC object
            digest = hashing.digest()

            # Get the last byte of the digest to extract the offset value
            offset = digest[-1] & 0x0f

            # Extract a 4-byte slice from the digest starting from the calculated offset
            truncated_hash = digest[offset:offset + 4]

            # Convert the truncated hash to an integer using big-endian byte order
            truncated_hash_int = int.from_bytes(truncated_hash, "big")

            # Remove the most significant bit by applying a bitwise AND operation
            truncated_hash_int &= 0x7fffffff

            if self.digits == 8:
                otp = truncated_hash_int % 100000000
            elif self.digits == 6:
                otp = truncated_hash_int % 1000000
            
            # Convert the OTP to a string
            totp = str(otp).zfill(self.digits)

            # Return the generated 6-digit OTP
            return totp
        
        except Exception as err:
            print(err)
            return None

    def get_remaining_time(self):
        return self.period - int(time.time() % self.period)


# __main__.py

import sys
from pyttm.libs.helper import *
from pyttm.libs.crypto import *


def main():
    try:
        clear()
        check_for_updates()
        command_line_args = sys.argv[1:]
        if len(command_line_args) == 0 or command_line_args[0] not in ["add", "list", "delete"]:
            print("Usage: ttm add|list|delete\n")
            exit()

        password = get_password()
        command = command_line_args[0]

        if command == "add":
            add_creds(password)
        elif command == "list":
            list_creds(password)
        elif command == "delete":
            delete_creds(password)

    except KeyboardInterrupt:
        print("\nExiting...")
        exit()
    except ValueError:
        print("Invalid input!")
        exit()
    except Exception as e:
        print("Error! Facing Issue, Please Report!")
        print(e)
        exit()


if __name__ == "__main__":
    main()



import os
from pathlib import Path
import sqlite3
from typing import Optional, TypedDict

from pyttm.libs.crypto import decrypt_text, encrypt_text


class Credentials(TypedDict):
    issuer: str
    seed: str
    algorithm: str
    digits: int
    period: int


class SQLiteDB:
    def __init__(self):
        self.create_table()

    @property
    def ttm_dir(self):
        if os.name == 'nt':
            config_dir = os.environ.get(
                'APPDATA', Path.home().joinpath('AppData', 'Roaming'))

        elif os.name == 'posix':
            config_dir = os.environ.get(
                'XDG_CONFIG_HOME', Path.home().joinpath('.config'))

        else:
            config_dir = Path(__file__).parent.parent

        return Path(config_dir).joinpath('ttm')

    @property
    def config_db(self):
        os.makedirs(self.ttm_dir, exist_ok=True)
        return Path(self.ttm_dir).joinpath('config.db')

    def create_table(self):
        with sqlite3.connect(self.config_db) as connection:
            cursor = connection.cursor()
            cursor.execute('''CREATE TABLE IF NOT EXISTS Password (
                id INTEGER PRIMARY KEY AUTOINCREMENT,
                password TEXT
            )''')

            cursor.execute('''CREATE TABLE IF NOT EXISTS Version (
                id INTEGER PRIMARY KEY AUTOINCREMENT,
                last_check DATETIME
            )''')

            cursor.execute('''CREATE TABLE IF NOT EXISTS Credentials (
                id INTEGER PRIMARY KEY AUTOINCREMENT,
                issuer TEXT,
                seed TEXT,
                algorithm TEXT,
                digits INTEGER,
                period INTEGER
            )''')

    @property
    def password_hash(self) -> Optional[str]:
        with sqlite3.connect(self.config_db) as connection:
            cursor = connection.cursor()
            cursor.execute("SELECT password FROM Password ORDER BY id DESC")
            result = cursor.fetchone()
            if result:
                return result[0]

    @password_hash.setter
    def password_hash(self, password: str):
        with sqlite3.connect(self.config_db) as connection:
            cursor = connection.cursor()
            cursor.execute(
                "INSERT INTO Password (password) VALUES (?)", (password,))

    def get_creds(self, password: str):
        with sqlite3.connect(self.config_db) as connection:
            cursor = connection.cursor()
            cursor.execute("SELECT * FROM Credentials")
            rows = cursor.fetchall()
            creds = []
            for row in rows:
                cred = {
                    'issuer': row[1],
                    'seed': decrypt_text(row[2], password),
                    'algorithm': row[3],
                    'digits': row[4],
                    'period': row[5]
                }
                creds.append(cred)
            return creds

    def add_creds(self, creds: Credentials, password: str):
        with sqlite3.connect(self.config_db) as connection:
            cursor = connection.cursor()
            encrypted_seed = encrypt_text(creds['seed'], password)
            cursor.execute("INSERT INTO Credentials (issuer, seed, algorithm, digits, period) VALUES (?, ?, ?, ?, ?)",
                           (creds['issuer'], encrypted_seed, creds['algorithm'], creds['digits'], creds['period']))

    def delete_cred(self, cred_id: int):
        with sqlite3.connect(self.config_db) as connection:
            cursor = connection.cursor()
            cursor.execute("DELETE FROM Credentials WHERE id=?", (cred_id,))

    def get_last_check(self):
        with sqlite3.connect(self.config_db) as connection:
            cursor = connection.cursor()
            cursor.execute("SELECT last_check FROM Version ORDER BY id DESC")
            result = cursor.fetchone()
            if result:
                return result[0]
        
    def set_last_check(self, last_check: str):
        with sqlite3.connect(self.config_db) as connection:
            cursor = connection.cursor()
            cursor.execute("INSERT INTO Version (last_check) VALUES (?)", (last_check,))

ConfigDB = SQLiteDB()



import pyttm.libs.crypto as crypto
import unittest

class TestEncryptionFunctions(unittest.TestCase):

    def test_encrypt_text(self):
        text = "Hello, World!"
        secret_key = "mysecretkey"     
        encrypted_text = crypto.encrypt_text(text, secret_key)
        decrypted_text = crypto.decrypt_text(encrypted_text, secret_key)
        self.assertEqual(decrypted_text, text)

    def test_decrypt_text(self):
        text = "Hello, World!"
        secret_key = "mysecretkey"
        encrypted_text = crypto.encrypt_text(text, secret_key)
        decrypted_text = crypto.decrypt_text(encrypted_text, secret_key)
        self.assertEqual(decrypted_text, text)

    def test_hash_password(self):
        password = "mysecretpassword"
        hashed_password = crypto.hash_password(password)
        self.assertNotEqual(hashed_password, "")

    def test_validate_password(self):
        password = "mysecretpassword"
        hashed_password = crypto.hash_password(password)
        self.assertTrue(crypto.validate_password(password, hashed_password))
        self.assertFalse(crypto.validate_password("wrongpassword", hashed_password))

if __name__ == '__main__':
    unittest.main()









import os
import urllib.parse
import requests
import pkg_resources
from time import sleep
from datetime import datetime
from pyttm.libs.db import ConfigDB
from pyttm.libs.crypto import *
from pyttm.libs.totp import TOTPGenerator


def get_password():
    password_hash = ConfigDB.password_hash
    if not password_hash:
        print("Welcome to TOTP Manager!\n")
        print("Create a password to secure your credentials.")
        while True:
            password = input("\nPassword: ")
            if (len(password) < 8):
                clear()
                print("Password must be at least 8 characters!")
                continue
            confirm_password = input("Confirm password: ")
            if password == confirm_password:
                break
            else:
                clear()
                print("Password does not match!")
        ConfigDB.password_hash = hash_password(password)
    else:
        while True:
            password = input("Enter password: ")
            if validate_password(password, ConfigDB.password_hash):
                break
            else:
                clear()
                print("Invalid password!")
    return password


def add_creds(password):
    if (input("Do you have otpauth uri? (y/n): ") in ["y", "Y"]):
        link = input("Enter otpauth link: ")
        totp_info = extract_info_from_otpauth_uri(link)
    else:
        totp_info = gather_info_from_user()
    ConfigDB.add_creds(totp_info, password)
    print("\nAdded Successfully!")


def list_creds(password):
    while True:
        clear()
        creds = ConfigDB.get_creds(password)
        if len(creds) == 0:
            print("No Credentials Found!")
            if (input("Do you want to add? (y/n): ") in ["y", "Y"]):
                add_creds(password)
                break
            exit()
        max_key_length = max(len(cred["issuer"]) for cred in creds)
        print(f"Issuer{' '*(max_key_length-5)}    OTP     Time")
        print("-"*(max_key_length+19))
        for cred in creds:
            i = TOTPGenerator(**cred)
            padding = ' ' * (max_key_length - len(cred["issuer"]))
            print(
                f"{cred['issuer']}{padding}    {i.generateTOTP()}    {i.get_remaining_time()}")
        print("\n\nPress Ctrl+C to exit")
        sleep(1)
        print("\n")


def delete_creds(password):
    clear()
    creds = ConfigDB.get_creds(password)
    if len(creds) == 0:
        print("No Credentials Found!")
        if (input("Do you want to add? (y/n): ") in ["y", "Y"]):
            add_creds(password)
        exit()
    else:
        print(f'Issuer\n{"-" * 6}')
        for index, cred in enumerate(creds):
            print(f"{index + 1}. {cred['issuer']}")
        choice = int(input("\nChoose the Issuer to delete: "))
        if choice > len(creds) or choice < 1:
            print("Invalid choice!")
            exit()
        if (input(f"Are you sure you want to delete {creds[choice-1]['issuer']}? (y/n): ") in ["y", "Y"]):
            ConfigDB.delete_cred(choice)
            print("Deleted Successfully!")
        else:
            print("Aborted!")
            exit()


def clear():
    if os.name == "nt":
        os.system("cls")
    else:
        os.system("clear")
    display_banner()


def display_banner():
    print("""
 -------------------------- 
|  _____   _____   __  __  |
| |_   _| |_   _| |  \/  | |
|   | |     | |   | |\/| | |
|   | |     | |   | |  | | |
|   |_|     |_|   |_|  |_| |
|                          |  
|   Terminal TOTP Manager  |
|      version : 0.1.0     |
 --------------------------
""")


def extract_info_from_otpauth_uri(uri):
    parsed_uri = urllib.parse.urlparse(uri)

    if parsed_uri.scheme != "otpauth" or parsed_uri.netloc != "totp":
        raise ValueError("Invalid OTPAuth URI")

    label = parsed_uri.path.strip('/')
    query_params = urllib.parse.parse_qs(parsed_uri.query)

    if 'secret' not in query_params:
        raise ValueError("Secret parameter not found in OTPAuth URI")

    secret = query_params['secret'][0]
    algorithm = query_params['algorithm'][0] if 'algorithm' in query_params else 'sha1'
    digits = query_params['digits'][0] if 'digits' in query_params else 6
    period = query_params['period'][0] if 'period' in query_params else 30
    label = f'{query_params["issuer"][0]} {label}' if 'issuer' in query_params else label

    return {'issuer': label, 'seed': secret, 'algorithm': algorithm, 'digits': digits, 'period': period}


def gather_info_from_user():
    issuer = input("Enter Issuer: ")
    seed = input("Enter Secret Key: ")
    algorithm = input("Enter Algorithm (Default: sha1): ")
    digits = input("Enter digits (Default: 6): ")
    period = input("Enter period (Default: 30): ")
    if algorithm == "":
        algorithm = "sha1"
    if digits == "":
        digits = 6
    if period == "":
        period = 30
    return {
        "issuer": issuer,
        "seed": seed,
        "algorithm": algorithm,
        "digits": digits,
        "period": period
    }

def check_for_updates():
    last_checked = ConfigDB.get_last_check()
    if last_checked:
        if (datetime.now() - datetime.strptime(last_checked, "%Y-%m-%d %H:%M:%S.%f")).days < 1:
            return
    ConfigDB.set_last_check(datetime.now())
    try:
        fetch = requests.get("https://pypi.org/pypi/pyttm/json")
        latest_version = fetch.json()["info"]["version"]
        current_version = pkg_resources.get_distribution("pyttm").version
        if latest_version != current_version :
            print(f"\nUpdate available: {latest_version}")
            print("Run 'pip install --upgrade pyttm' to update\n")
    except:
        pass


from setuptools import setup , find_packages
import os

def docs_read(fname):
    f = open(os.path.join(os.path.dirname(__file__), fname)).read()
    return f.replace('â™¥', '♥')

setup(
    name='pyttm',
    version='0.1.0',
    description='A simple Terminal based TOTP manager',
    long_description=(docs_read('README.md')),
    long_description_content_type='text/markdown',
    url='https://github.com/The-Robin-Hood/ttm',
    author='The Robin Hood',
    license='MIT',
    platforms=['any'],
    entry_points={
        'console_scripts': [
            'ttm = pyttm.app:main'
        ],
    },
    packages=find_packages(where='src',include=['pyttm','pyttm.*']),
    package_dir={'': 'src'},
    include_package_data=True,
    install_requires=['pycryptodome'],
    keywords='totp,otp,encryption,decryption,AES256,crypto,security,privacy',
        classifiers=[
        'Intended Audience :: Developers',
        'Topic :: Security',
        'Topic :: Utilities',
        'Development Status :: 4 - Beta',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Unix',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python :: 3'
    ],
)